import { createStore } from 'vuex'
import { v4 as uuidv4 } from "uuid"
import { instance } from "./axios"


export default createStore({
  state: {
    projects: [],
    files: [],
    user: null,
    errors: [],
    previewFile: null,
    uploads: []
  },

  mutations: {
    addError(state, message) {
      let error = {'id': uuidv4(), 'message': message}
      state.errors.push(error)
    },
    removeError(state, error) {
      state.errors = state.errors.filter(err => err.id != error.id )
    },
    clearErrors(state) {
      state.errors = []
    },
    setUser(state, user) {
      state.user = user
    },
    setProjects(state, projects) {
      state.projects = projects
    },
    setFiles(state, files) {
      state.files = files
    },
    setPreviewFile(state, file) {
      state.previewFile = file
    },
    addUpload(state, upload) {
      state.uploads.push(upload)
    },
    updateUpload(state, upload) {
      for (const up of state.uploads) {
        if (up.id == upload.id) {
          up.progress = upload.progress
          break
        }
      }
    },
    removeUpload(state, uploadID) {
      state.uploads = state.uploads.filter(up => up.id != uploadID )
    },
  },
  getters: {
    isAuthenticated: state => !!state.user
  },
  actions: {
    async login({commit}, auth) {
      try {
        const response = await instance.get('/users/')
        let user_object = response.data.results[0]
        user_object.auth = auth
        commit('setUser', user_object)
      } catch(err) {
        commit('setUser', null)
        commit('addError', JSON.stringify(err.response.data))
      }
    },
    async loadProjects({commit}) {
      try {
        const response = await instance.get('/projects/')
        commit('setProjects', response.data.results)
      } catch(err) {
        commit('addError', JSON.stringify(err.response.data))
      }
    },
    async getProject({commit}, project_id) {
      try {
        return instance.get(`projects/${project_id}/`)
      } catch(err) {
        commit('addError', JSON.stringify(err.response.data))
      }
    },
    async createProject({dispatch, commit}, payload) {
      try {
        await instance.post('/projects/', payload)
        dispatch('loadProjects')
      } catch(err) {
        commit('addError', JSON.stringify(err.response.data))
      }
    },
    async deleteProject({commit, dispatch}, project_id) {
      try {
        await instance.delete(`projects/${project_id}/`)
        dispatch('loadProjects')
      } catch(err) {
        commit('addError', JSON.stringify(err.response.data))
      }
    },
    async loadFiles({commit}, project_id) {
      try {
        const response = await instance.get(`projects/${project_id}/files/`)
        commit('setFiles', response.data.results)
      } catch(err) {
        commit('addError', JSON.stringify(err.response.data))
      }
    },
    async createFile({dispatch, commit}, payload) {
      let uploadID = uuidv4()
      let uploadObject = { 'id': uploadID, progress: 0 }
      commit('addUpload', uploadObject)

      const config = {
        onUploadProgress: progressEvent => {
          let progress = (progressEvent.loaded / progressEvent.total) * 100;
          let uploadObject = { 'id': uploadID, progress }
          commit('updateUpload', uploadObject)
        }
      }

      try {
        await instance.post(`${payload.project_url}files/`, payload.data, config)
        commit('removeUpload', uploadID)
        dispatch('loadFiles', payload.project_id)
      } catch(err) {
        commit('removeUpload', uploadID)
        commit('addError', JSON.stringify(err.response.data))
      }
    }
  },
  modules: {
  }
})
